@@ -105,7 +105,7 @@ static void indirect_in_tree (struct buffer_head * bh,
     unp = (__u32 *)B_I_PITEM (bh, ih);
     
     for (i = 0; i < I_UNFM_NUM (ih); i ++) {
-	unfm_ptr = le32_to_cpu (unp[i]);
+	unfm_ptr = d32_get (unp, i);
 	if (unfm_ptr == 0)
 	    continue;
 	if ((ret = still_bad_unfm_ptr_1 (unfm_ptr)))
@@ -349,11 +349,13 @@ static void pass1_correct_leaf (reiserfs_filsys_t * fs,
     for (i = 0; i < B_NR_ITEMS (bh); i ++, ih ++) {
 	if (is_direntry_ih (ih)) {
 	    struct reiserfs_de_head * deh;
+	    __u32 offset;
 	    char * name;
 	    int name_len;
 	    unsigned int hash_code;
 
 	    deh = B_I_DEH (bh, ih);
+	    offset = 0;
 	    for (j = 0; j < get_ih_entry_count (ih); j ++) {
 		name = name_in_entry (deh + j, j);
 		name_len = name_in_entry_length (ih, deh + j, j);
@@ -363,8 +365,7 @@ static void pass1_correct_leaf (reiserfs_filsys_t * fs,
 		    continue;
 		}
 
-		hash_code = find_hash_in_use (name, name_len,
-					      GET_HASH_VALUE (get_deh_offset (deh + j)),
+		hash_code = find_hash_in_use (name, name_len, get_deh_offset (deh + j),
 					      get_sb_hash_code (fs->fs_ondisk_sb));
 		if (hash_code != get_sb_hash_code (fs->fs_ondisk_sb)) {
 		    fsck_log ("pass1: block %lu, item %d, entry %d: The entry \"%.*s\" of the %k is hashed with %s "
@@ -381,8 +382,24 @@ static void pass1_correct_leaf (reiserfs_filsys_t * fs,
 			fsck_log(" - deleted\n");
 			j --;
 			deh = B_I_DEH (bh, ih);
+			continue;
 		    }
 		}
+
+		if (j && offset >= get_deh_offset (deh + j)) {
+		    fsck_log ("pass1: block %lu, item %d, entry %d: The entry "
+			      "\"%.*s\" of the %k has hash offset %lu not "
+			      "larger smaller than the previous one %lu. The "
+			      "entry is deleted.\n", bh->b_blocknr, 
+			      i, j, name_len, name, &ih->ih_key, 
+			      get_deh_offset(deh + j), offset);
+		    cut_entry (fs, bh, i, j, 1);
+		    j --;
+		    deh = B_I_DEH (bh, ih);
+		    continue;
+		}
+
+		offset = get_deh_offset (deh + j);
 	    }
 	    continue;
 	}
@@ -394,8 +411,8 @@ static void pass1_correct_leaf (reiserfs_filsys_t * fs,
 	/* correct indirect items */
 	ind_item = (__u32 *)B_I_PITEM (bh, ih);
 
-	for (j = 0; j < I_UNFM_NUM (ih); j ++, ind_item ++) {
-	    unfm_ptr = le32_to_cpu (*ind_item);
+	for (j = 0; j < I_UNFM_NUM (ih); j ++) {
+	    unfm_ptr = d32_get (ind_item, j);
 
 	    if (!unfm_ptr)
 		continue;
@@ -409,7 +426,7 @@ static void pass1_correct_leaf (reiserfs_filsys_t * fs,
 	    /* 1. zero slots pointing to a leaf */
 	    if (is_used_leaf (unfm_ptr)) {
 		dirty ++;
-		*ind_item = 0;
+		d32_put (ind_item, j, 0);
 		pass_1_stat (fs)->pointed_leaves ++;
 		continue;
 	    }
@@ -424,7 +441,7 @@ static void pass1_correct_leaf (reiserfs_filsys_t * fs,
 		else {
 		    /* Yes, we have seen this pointer already, zero other pointers to it. */
 		    dirty ++;
-		    *ind_item = 0;
+		    d32_put (ind_item, j, 0);
 		    pass_1_stat (fs)->non_unique_pointers ++;
 		    continue;
 		}
