@@ -1,11 +1,13 @@
 /*
- * Copyright 2001-2003 by Hans Reiser, licensing governed by 
+ * Copyright 2001-2004 by Hans Reiser, licensing governed by 
  * reiserfsprogs/README
  */
 
 #include "fsck.h"
-#include <stdlib.h>
-#include <errno.h>
+
+#if defined(HAVE_LIBUUID) && defined(HAVE_UUID_UUID_H)
+#    include <uuid/uuid.h>
+#endif
 
 #define fsck_conditional_log(sb_found, fmt, list...) {	\
     if (sb_found)					\
@@ -186,9 +188,15 @@ void rebuild_sb (reiserfs_filsys_t * fs, char * filename, struct fsck_data * dat
 
     char * answer = 0;
     size_t n = 0;
-    struct stat64 stat_buf;
+    struct stat stat_buf;
     int retval, exit_code = EXIT_OK;
 
+#if defined(HAVE_LIBUUID) && defined(HAVE_UUID_UUID_H)
+    char uuid[37];
+    
+    uuid[36] = '\0';
+#endif
+
     if (!no_reiserfs_found (fs)) {
         sb = getmem (sizeof (*sb));
         if (!is_opened_rw (fs)) {
@@ -212,7 +220,8 @@ void rebuild_sb (reiserfs_filsys_t * fs, char * filename, struct fsck_data * dat
 	    fs->fs_blocksize = retval;
 	}
 
-        block_count = count_blocks (filename, fs->fs_blocksize);
+        if (!(block_count = count_blocks (filename, fs->fs_blocksize)))
+		exit(EXIT_OPER);
 
         /* save ondisk_sb somewhere and work in temp area */
         ondisk_sb = fs->fs_ondisk_sb;
@@ -305,13 +314,12 @@ void rebuild_sb (reiserfs_filsys_t * fs, char * filename, struct fsck_data * dat
                 getline (&answer, &n, stdin);
                 if (strcmp ("\n", answer))
                     block_count = atoi (answer);
-                set_sb_block_count (sb, block_count);
             } else {
                 fsck_conditional_log (magic_was_found, "rebuild-sb: wrong block count "
 		    "occured (%lu), fixed (%lu)\n", get_sb_block_count(sb), block_count);
-
-                set_sb_block_count (sb, block_count);
             }
+
+	    set_sb_block_count (sb, block_count);
         }
 
         if (get_sb_block_size (sb) != fs->fs_blocksize) {
@@ -351,7 +359,8 @@ void rebuild_sb (reiserfs_filsys_t * fs, char * filename, struct fsck_data * dat
 		retval = 4096;
 	}
 
-        block_count = count_blocks (filename, retval);
+        if (!(block_count = count_blocks (filename, retval)))
+		exit(EXIT_OPER);
 
         switch(version){
         case 1:
@@ -491,14 +500,13 @@ void rebuild_sb (reiserfs_filsys_t * fs, char * filename, struct fsck_data * dat
     }
 
     if (version == 1 || version == 3) {
-        if (!uuid_is_correct(sb->s_uuid)) {
-	    if (generate_random_uuid (sb->s_uuid)) {
-		fsck_log ("rebuild-sb: no uuid found, failed to genetate UUID\n");
-	    } else {
-		fsck_log ("rebuild-sb: no uuid found, a new uuid generated (%U)\n", 
-		    sb->s_uuid);
-	    }
+#if defined(HAVE_LIBUUID) && defined(HAVE_UUID_UUID_H)
+        if (uuid_is_null(sb->s_uuid)) {
+	    uuid_generate(sb->s_uuid);
+	    fsck_log ("rebuild-sb: no uuid found, a new uuid was "
+		      "generated (%U)\n", sb->s_uuid);
         }
+#endif
 	if (sb->s_flags != 0 && sb->s_flags != 1) {
 	    fsck_log ("rebuild-sb: super block flags found (%u), zeroed\n",
                   sb->s_flags);
@@ -557,16 +565,16 @@ void rebuild_sb (reiserfs_filsys_t * fs, char * filename, struct fsck_data * dat
         }
 
 	if ((retval = reiserfs_open_journal(fs, filename, O_RDWR | O_LARGEFILE))) {
-	    fsck_log("\nrebuild-sb: Failed to open the journal device (%s).", 
+	    fsck_log("\nrebuild-sb: Failed to open the journal device (%s).\n", 
 		filename);
 
-	    exit(retval == 1 ? EXIT_USER : EXIT_OPER);
+	    exit(retval < 0 ? EXIT_OPER : EXIT_USER);
 	}	
     } else if (!fsck_skip_journal(fs)) {
 	/* Check that specified non-standard journal device exists. */
 	    
 	journal_dev_name = fsck_data (fs)->journal_dev_name;	    
-	retval = stat64(journal_dev_name, &stat_buf);
+	retval = stat(journal_dev_name, &stat_buf);
 	    
 	if (retval == -1)
 	    reiserfs_exit (EXIT_USER, "rebuild_sb: error while detecting the "
@@ -595,8 +603,8 @@ void rebuild_sb (reiserfs_filsys_t * fs, char * filename, struct fsck_data * dat
 	    }
 
 	    if (!reiserfs_journal_opened(fs)) {
-		fsck_log ("Journal cannot be opened, assuming specified journal device "
-			"is correct\n");
+		fsck_log ("Journal cannot be opened, assuming specified "
+			  "journal device is correct\n");
 	    }
 	}
 	
@@ -623,7 +631,9 @@ void rebuild_sb (reiserfs_filsys_t * fs, char * filename, struct fsck_data * dat
             } else
                 set_jp_journal_1st_block (sb_jp(sb), default_value);
 
-            p_jp_dev_size = count_blocks (journal_dev_name, fs->fs_blocksize);
+            if (!(p_jp_dev_size = count_blocks (journal_dev_name, fs->fs_blocksize)))
+		    exit(EXIT_OPER);
+
             /* some checks for journal offset */
             if (strcmp(fs->fs_file_name, journal_dev_name) != 0) {
                 if (p_jp_dev_size < get_jp_journal_1st_block (sb_jp(sb)) + 1)
@@ -674,7 +684,7 @@ void rebuild_sb (reiserfs_filsys_t * fs, char * filename, struct fsck_data * dat
 		fsck_log("\nrebuild-sb: Failed to open a journal device (%s).", 
 		    journal_dev_name);
 		
-		exit(retval == 1 ? EXIT_USER : EXIT_OPER);
+		exit(retval < 0 ? EXIT_OPER : EXIT_USER);
 	    }
 	
 	    /* SB was found, but journal params were broken and have been recovered. 
@@ -760,8 +770,6 @@ void rebuild_sb (reiserfs_filsys_t * fs, char * filename, struct fsck_data * dat
 		set_jp_journal_magic (&j_head->jh_journal, magic);
 
 	    }
-	} else {
-	    set_jp_journal_magic (sb_jp(sb), 0);	    
 	}
 	
 	retval = memcmp(&j_head->jh_journal, sb_jp(sb), sizeof(struct journal_params));
