@@ -318,6 +318,7 @@ int replay_one_transaction (reiserfs_filsys_t * fs,
 			    reiserfs_trans_t * trans)
 {
     for_each_block (fs, trans, read_journal_write_in_place);
+    fsync(fs->fs_dev);
     return 0;
 }
 
@@ -500,25 +501,37 @@ int reiserfs_open_journal (reiserfs_filsys_t * fs, char * j_filename, int flags)
 	    "specified journal device %s.\nMust be not less than (%lu).\n",
 	    get_jp_journal_size (sb_jp (sb)) + 1, j_filename, 
 	    JOURNAL_MIN_SIZE + 1);
+	close(fs->fs_journal_dev);
 	return 1;
     }
     
-    count = count_blocks (j_filename, fs->fs_blocksize);
-    if (get_jp_journal_1st_block (sb_jp (sb)) + get_jp_journal_size (sb_jp (sb)) + 1 > count) {
-	reiserfs_warning (stderr, "Detected journal on specified device %s does not fit to "
-	    "the device.\nStart block (%lu) + size (%lu) less than device size (%lu).\n", 
-	    j_filename, get_jp_journal_1st_block(sb_jp (sb)), 
-	    get_jp_journal_size(sb_jp (sb)) + 1, count);
+    if (!(count = count_blocks (j_filename, fs->fs_blocksize))) {
+	close(fs->fs_journal_dev);
+	return -1;
+    }
+
+    if (get_jp_journal_1st_block (sb_jp (sb)) + 
+	get_jp_journal_size (sb_jp (sb)) + 1 > count) 
+    {
+	reiserfs_warning (stderr, "Detected journal on specified device %s "
+			  "does not fit to the device.\nStart block (%lu) + "
+			  "size (%lu) less than device size (%lu).\n", 
+			  j_filename, get_jp_journal_1st_block(sb_jp (sb)), 
+			  get_jp_journal_size(sb_jp (sb)) + 1, count);
+	close(fs->fs_journal_dev);
 	return 1;
     }
     
     /* read journal header */
-    fs->fs_jh_bh = bread (fs->fs_journal_dev, get_jp_journal_1st_block (sb_jp (sb)) 
-	+ get_jp_journal_size (sb_jp (sb)), fs->fs_blocksize);
+    fs->fs_jh_bh = bread (fs->fs_journal_dev, 
+			  get_jp_journal_1st_block (sb_jp (sb)) + 
+			  get_jp_journal_size (sb_jp (sb)), 
+			  fs->fs_blocksize);
 
     if (!fs->fs_jh_bh) {
-	reiserfs_warning (stderr, "reiserfs_open_journal: bread failed reading "
-	    "journal  header.\n");
+	reiserfs_warning (stderr, "reiserfs_open_journal: bread failed "
+			  "reading journal  header.\n");
+	close(fs->fs_journal_dev);
 	return -1;
     }
 
@@ -573,7 +586,9 @@ int reiserfs_create_journal(
 	st.st_rdev = 0;
     } else {
 	/* journal is to be on separate device */
-	blocks = count_blocks (j_device, fs->fs_blocksize);
+	if (!(blocks = count_blocks (j_device, fs->fs_blocksize)))
+		return 0;
+
 	if (!len) {
 	    /* default size of a journal on a separate device is whole device */
 	    if (blocks < offset) {
@@ -734,17 +749,21 @@ void reiserfs_close_journal (reiserfs_filsys_t * fs)
 }
 
 /* update journal header */
-static void update_journal_header (struct buffer_head * bh_jh, reiserfs_trans_t trans) {
+static void update_journal_header (reiserfs_filsys_t * fs, 
+				   struct buffer_head * bh_jh, 
+				   reiserfs_trans_t *trans) 
+{
     struct reiserfs_journal_header * j_head;
 	
     j_head = (struct reiserfs_journal_header *)(bh_jh->b_data);
 
     /* update journal header */
-    set_jh_last_flushed (j_head, trans.trans_id);
-    set_jh_mount_id (j_head, trans.mount_id);
-    set_jh_replay_start_offset (j_head, trans.next_trans_offset);
+    set_jh_last_flushed (j_head, trans->trans_id);
+    set_jh_mount_id (j_head, trans->mount_id);
+    set_jh_replay_start_offset (j_head, trans->next_trans_offset);
     mark_buffer_dirty (bh_jh);
     bwrite (bh_jh);
+    fsync(fs->fs_journal_dev);
 }
 
 /* fixme: what should be done when not all transactions can be replayed in proper order? */
@@ -808,7 +827,7 @@ int replay_journal (reiserfs_filsys_t * fs)
 	    cur.mount_id, cur.trans_id, cur.desc_blocknr, cur.trans_len,
 	    cur.commit_blocknr, cur.next_trans_offset);
 	replay_one_transaction (fs, &cur);
-	update_journal_header (bh, cur);
+	update_journal_header (fs, bh, &cur);
 	control = cur;
         replayed ++;
 
@@ -825,7 +844,7 @@ int replay_journal (reiserfs_filsys_t * fs)
     mark_buffer_dirty (fs->fs_super_bh);
     bwrite (fs->fs_super_bh);
 	
-    update_journal_header (bh, newest);
+    update_journal_header (fs, bh, &newest);
 
     return 0;
 }
