@@ -1,10 +1,11 @@
 /*
- * Copyright 2000-2002 by Hans Reiser, licensing governed by reiserfs/README
+ * Copyright 2000-2004 by Hans Reiser, licensing governed by 
+ * reiserfsprogs/README
  */
+
 #include "resize.h"
 #include <time.h>
 
-
 static unsigned long int_node_cnt   = 0, int_moved_cnt   = 0;
 static unsigned long leaf_node_cnt  = 0, leaf_moved_cnt  = 0;
 static unsigned long unfm_node_cnt  = 0, unfm_moved_cnt  = 0;
@@ -24,19 +25,24 @@ static void quit_resizer(reiserfs_filsys_t * fs)
 	/* save changes to bitmap blocks */
 	reiserfs_close (fs);
 	/* leave fs in ERROR state */
-	DIE ("fs shrinking was not completed successfully, run reiserfsck.");
+	reiserfs_exit(1, "fs shrinking was not completed successfully, "
+		      "run reiserfsck.");
 }
 
 /* block moving */
-static unsigned long move_generic_block(reiserfs_filsys_t * fs, unsigned long block, unsigned long bnd, int h)
+static unsigned long move_generic_block(reiserfs_filsys_t * fs, 
+					unsigned long block, 
+					unsigned long bnd, int h)
 {
     struct buffer_head * bh, * bh2;
 
 	/* primitive fsck */
 	if (block > get_sb_block_count(ondisk_sb)) {
-		fprintf(stderr, "resize_reiserfs: invalid block number (%lu) found.\n", block);
+		fprintf(stderr, "resize_reiserfs: invalid block number "
+			"(%lu) found.\n", block);
 		quit_resizer(fs);
 	}
+	
 	/* progress bar, 3D style :) */
 	if (opt_verbose)
 	    print_how_far(stderr, &total_node_cnt, blocks_used, 1, 0);
@@ -56,7 +62,7 @@ static unsigned long move_generic_block(reiserfs_filsys_t * fs, unsigned long bl
 	bh = bread(fs->fs_dev, block, fs->fs_blocksize);
 
 	if (!bh)
-	    reiserfs_panic ("move_generic_block: bread failed.\n");
+	    reiserfs_exit (1, "move_generic_block: bread failed.\n");
 
 	reiserfs_bitmap_find_zero_bit(bmp, &unused_block);
 	if (unused_block == 0 || unused_block >= bnd) {
@@ -98,18 +104,24 @@ static unsigned long move_formatted_block(reiserfs_filsys_t * fs, unsigned long
 	struct item_head *ih;
 	unsigned long new_blocknr = 0;
 	int node_is_internal = 0;
-	int i, j;
+	unsigned int i, j;
 	
 	bh = bread(fs->fs_dev, block, fs->fs_blocksize);
 	if (!bh)
-	    reiserfs_panic ("move_formatted_block: bread failed");
+	    reiserfs_exit (1, "move_formatted_block: bread failed");
 	
 	if (is_leaf_node (bh)) {
 		
 		leaf_node_cnt++;
 		
-		for (i=0; i < B_NR_ITEMS(bh); i++) {
+		for (i = 0; i < B_NR_ITEMS(bh); i++) {
 			ih = B_N_PITEM_HEAD(bh, i);
+			
+			/* skip the bad blocks. */
+			if (get_key_objectid (&ih->ih_key) == BADBLOCK_OBJID &&
+			    get_key_dirid (&ih->ih_key) == BADBLOCK_DIRID)
+				continue;
+			
 			if (is_indirect_ih(ih)) {
 				__u32 * indirect;
 
@@ -117,11 +129,11 @@ static unsigned long move_formatted_block(reiserfs_filsys_t * fs, unsigned long
 				for (j = 0; j < I_UNFM_NUM(ih); j++) {
 					unsigned long  unfm_block;
 
-					if (indirect [j] == 0) /* hole */
+					if (d32_get (indirect, j) == 0) /* hole */
 						continue;
-					unfm_block = move_unformatted_block(fs, le32_to_cpu (indirect [j]), bnd, h + 1);
+					unfm_block = move_unformatted_block(fs, d32_get (indirect, j), bnd, h + 1);
 					if (unfm_block) {
-						indirect [j] = cpu_to_le32 (unfm_block);
+						d32_put (indirect, j, unfm_block);
 						mark_buffer_dirty(bh);
 					}
 				}
@@ -162,41 +174,46 @@ static unsigned long move_formatted_block(reiserfs_filsys_t * fs, unsigned long
 	return new_blocknr;
 }
 
-int shrink_fs(reiserfs_filsys_t * fs, unsigned long blocks)
+int shrink_fs(reiserfs_filsys_t * fs, long long int blocks)
 {
 	unsigned long n_root_block;
 	unsigned int bmap_nr_new;
-
+	unsigned long bad_count;
 	
 	ondisk_sb = fs->fs_ondisk_sb;
 	
+	
+	bmap_nr_new = (blocks - 1) / (8 * fs->fs_blocksize) + 1;
+
+	/* is shrinking possible ? */
+	if (get_sb_block_count(ondisk_sb) - blocks > 
+	    get_sb_free_blocks(ondisk_sb) + get_sb_bmap_nr(ondisk_sb) - 
+	    bmap_nr_new) 
+	{
+	    fprintf(stderr, "resize_reiserfs: can\'t shrink fs; too many "
+		"blocks already allocated\n");
+	    return -1;
+	}
+
 	/* warn about alpha version */
 	{
 		int c;
 
 		printf(
-			"You are running BETA version of reiserfs shrinker.\n"
-			"This version is only for testing or VERY CAREFUL use.\n"
-			"Backup of you data is recommended.\n\n"
-			"Do you want to continue? [y/N]:"
-			);
+		    "You are running BETA version of reiserfs shrinker.\n"
+		    "This version is only for testing or VERY CAREFUL use.\n"
+		    "Backup of you data is recommended.\n\n"
+		    "Do you want to continue? [y/N]:"
+		);
 		fflush(stdout);
 		c = getchar();
 		if (c != 'y' && c != 'Y')
 			exit(1);
 	}
 
-	bmap_nr_new = (blocks - 1) / (8 * fs->fs_blocksize) + 1;
-
-	/* is shrinking possible ? */
-	if (get_sb_block_count(ondisk_sb) - blocks > get_sb_free_blocks(ondisk_sb) + get_sb_bmap_nr(ondisk_sb) - bmap_nr_new) {
-	    fprintf(stderr, "resize_reiserfs: can\'t shrink fs; too many blocks already allocated\n");
-	    return -1;
-	}
-
 	reiserfs_reopen(fs, O_RDWR);
 	if (reiserfs_open_ondisk_bitmap (fs))
-	    DIE("cannot open ondisk bitmap");
+	    reiserfs_exit(1, "cannot open ondisk bitmap");
 	bmp = fs->fs_bitmap2;
 	ondisk_sb = fs->fs_ondisk_sb;
 
@@ -221,10 +238,11 @@ int shrink_fs(reiserfs_filsys_t * fs, unsigned long blocks)
 		fflush(stdout);
 	}
 
-	n_root_block = move_formatted_block(fs, get_sb_root_block(ondisk_sb), blocks, 0);
-	if (n_root_block) {
-		set_sb_root_block (ondisk_sb, n_root_block);
-	}
+	n_root_block = move_formatted_block(fs, get_sb_root_block(ondisk_sb), 
+	    blocks, 0);
+	
+	if (n_root_block)
+	    set_sb_root_block (ondisk_sb, n_root_block);
 
 	if (opt_verbose)
 	    printf ("\n\nnodes processed (moved):\n"
@@ -243,7 +261,6 @@ int shrink_fs(reiserfs_filsys_t * fs, unsigned long blocks)
 		    (unsigned long)total_node_cnt, blocks_used);
 	}
 
-#if 1
 	{
 	    unsigned long l;
 
@@ -257,13 +274,23 @@ int shrink_fs(reiserfs_filsys_t * fs, unsigned long blocks)
 	    }
 	    printf("\n");
 	}
-#endif
 
+	badblock_list(fs, mark_badblock, NULL);
+	
+	if (fs->fs_badblocks_bm) {
+		bad_count = reiserfs_bitmap_ones(fs->fs_badblocks_bm);
+		reiserfs_shrink_bitmap (fs->fs_badblocks_bm, blocks);
+		add_badblock_list(fs, 1);
+		bad_count -= reiserfs_bitmap_ones(fs->fs_badblocks_bm);
+	} else
+		bad_count = 0;
+	
 	reiserfs_shrink_bitmap (fs->fs_bitmap2, blocks);
 
 	set_sb_free_blocks (ondisk_sb, get_sb_free_blocks(ondisk_sb)
 			    - (get_sb_block_count(ondisk_sb) - blocks)
-			    + (get_sb_bmap_nr(ondisk_sb) - bmap_nr_new));
+			    + (get_sb_bmap_nr(ondisk_sb) - bmap_nr_new)
+			    + bad_count);
 	set_sb_block_count (ondisk_sb, blocks);
 	set_sb_bmap_nr (ondisk_sb, bmap_nr_new);
 
